package org.vishia.MoneySim;

public class BuyingBehav {

  
  public final String good;
  
  public final String comment;
  
  /**This array is sorting to the price in {@link PointPriceBuyingAccept#price}
   * and contains a curve for the buying acceptance.
   * Both values are nominal values to 1.0. It means 1.0 for the price is the normal expected price for one unit.
   */
  public final PointPriceBuyingAccept[] buyingAccept;

  
  
  
  public BuyingBehav ( String good, String comment, PointPriceBuyingAccept[] buyingAccept
  ) {
    super();
    this.good = good;
    this.comment = comment;
    this.buyingAccept = buyingAccept;
  }

  @Override public String toString () {
    StringBuilder sb = new StringBuilder();
    sb.append(this.comment).append(": ").append(this.good);
    for(PointPriceBuyingAccept point: this.buyingAccept) {
      sb.append(" $").append(point.price).append("=").append(point.buyingAccept).append("; ");
    }
    return sb.toString();
  }


  static class PointPriceBuyingAccept {
    /**
     * Relative price on the market. 1.0 is related to the nominal (average) price for a nominal amount of a good.
     */
    final float price;
    /**
     * Acceptance to buy a good for the given price. It should be near 1.0 for the price 1.0,
     * can be fast get lower if a higher price is not accepted,
     * and get be higher for a lower price. But this depends of the quality of the good:
     * Is it able to store at home, make durable goods from it, or such. 
     */
    final float buyingAccept;
    public PointPriceBuyingAccept ( float price, float buyingAccept ) {
      this.price = price;
      this.buyingAccept = buyingAccept;
    }
  }
  
  
}
