package org.vishia.MoneySim;

/**
 * This class contains only a little bit common mathematics
 * <ul>
 * <li>{@link #getInterpolation(ValuePair[], float)} works with a array of points given with {@link ValuePair}.
 *   The [][0] values must be sorted (ascending or descending) because binary search is used to the x point.
 *   Left and right side from the x range is stop, returns always the same y value on first or last x position
 *   (means [0][1] and [lastIx][1]).
 *   The interpolation is yet linear (improvements plant).
 * <li>{@link #binarySearch(ValuePair[], float)} internal used for that.
 * <li>{@link #round2(float)} a small helper function to return the float nearest with 2 digits after decimal point.
 * </ul>  
 */
public class MCalc {

  
  public static float getInterpolation(MCalc.ValuePair[] table, float input) {
    int ix = binarySearch(table, input);
    if(ix <0) { return table[0].y; }
    if(ix < table.length -1) {
      int ix2 = ix +1;
      float y = (table[ix2].y - table[ix].y) / (table[ix2].x - table[ix].x)  * (input - table[ix].x) + table[ix].y;
      return y;
    } else {
      return table[ix].y;
    }
  }

  /**rounds to 2 digits after decimal point
   * @param val
   * @return
   */
  public static float round2(float val) {
    return ((int)(100 * val + 0.5f)) / 100.0f;
  }
  
  
  /**
   * @param table
   * @param x
   * @return the left point in the segment, -1 if x is left from segments, table.length-1 (last point) if x is right from segments.
   */
  private static int binarySearch(MCalc.ValuePair[] table, float x) {
    int low = 0;
    int zTable = table.length;
    boolean bAscending = table[0].x < table[zTable-1].x;
    if(bAscending ? x < table[0].x : x > table[0].x) return -1;
    if(bAscending ? x > table[zTable-1].x : x < table[zTable-1].x) return zTable-1;
    int high = zTable;
    while (low +1  < high) 
    {
      int mid = (low + high) >> 1;
      if(mid >=0) {
        float midVal = table[mid].x;
        if (bAscending ? midVal <= x : midVal >= x) { 
          low = mid;
        }
        else { 
          high = mid;  
        }
      } else {
        low = -1;
      }
    }
    return low;  // pos not found.
  }
  

  public static class ValuePair {
    float x; float y;

    public ValuePair(float x, float y) {
      super();
      this.x = x;
      this.y = y;
    }
    
  }
}
