package org.vishia.MoneySim;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.List;

import org.vishia.msgDispatch.LogMessage;
import org.vishia.msgDispatch.LogMessageStream;
import org.vishia.util.Debugutil;
import org.vishia.util.FileFunctions;

public class MainMoneySim {

  /**Version, history and license.
   * <ul>
   * <li>2025-02-26 first version 
   * </ul>
   * <br><br>
   * <b>Copyright/Copyleft</b>:
   * For this source the LGPL Lesser General Public License,
   * published by the Free Software Foundation is valid.
   * It means:
   * <ol>
   * <li> You can use this source without any restriction for any desired purpose.
   * <li> You can redistribute copies of this source to everybody.
   * <li> Every user of this source, also the user of redistribute copies
   *    with or without payment, must accept this license for further using.
   * <li> But the LPGL is not appropriate for a whole software product,
   *    if this source is only a part of them. It means, the user
   *    must publish this part of source,
   *    but don't need to publish the whole source of the own product.
   * <li> You can study and modify (improve) this source
   *    for own using or for redistribution, but you have to license the
   *    modified sources likewise under this LGPL Lesser General Public License.
   *    You mustn't delete this Copyright/Copyleft inscription in this source file.
   * </ol>
   * If you are intent to use this sources without publishing its usage, you can get
   * a second license subscribing a special contract with the author. 
   * 
   * @author Hartmut Schorrig = hartmut.schorrig@vishia.de
   */
  public final static String version = "2025-02-26"; 

  
  
  static class MainArgs extends org.vishia.util.Arguments {

    


    
    /**
     * This argument describes the mean value of fish per seller.
     */
    float amountFish = 250;
   
    long seedRandom = 1234;
    
    boolean bVerbatile;
    
    /**This array describes the commands with its help and stores the result to CmdArgs. More is not necessary. 
     */
    Argument[] argList1 =
      { new Argument("-a", ":250 - middle amount of fish (kg)"
      , new SetArgument() { @Override public boolean setArgument(String val) 
      { MainArgs.this.amountFish = Float.parseFloat(val); return true; } })
      , new Argument("-random", ":1234 - any value for random, influences random results."
          , new SetArgument() { @Override public boolean setArgument(String val) 
          { MainArgs.this.seedRandom = Integer.parseInt(val); return true; } })
      , new Argument("-versatile", " - versatile output on screen."
          , new SetArgument() { @Override public boolean setArgument(String val) 
          { MainArgs.this.bVerbatile = true; return true; } })
          
      };
   
    MainArgs(){
      super();
      super.aboutInfo = "Simulation of money " + MainMoneySim.version;
      super.helpInfo="all arguments are optional, output is written in output/out.txt";
      for(Argument arg: this.argList1) { addArg(arg); }
    }
    
    @Override public boolean testConsistence ( Appendable msg ) {
      
      return true;
    }
    
  }
  
  
  final LogMessage log, logAll;
  
  final MainArgs args;
  


  //@SuppressWarnings("resource") 
  FileOutputStream fsOut = null;  // will be close in finalize or at least in exec
  
  final ReadParameter readParameter;
  
  final CityData cityData;
  
  final EvalAndShow evalShow;
  
  /**Main entry from command line invocation.
   * @param sArgs The command line arguments as usual
   */
  static public void main(String[] sArgs) {
    try {
      int exitCode = smain(sArgs, System.out, System.err);
      System.exit(exitCode);
    } catch (Exception e) {
      System.err.println("Unexpected: " + e.getMessage());
      e.printStackTrace(System.err);
      System.exit(255);
    }
  }
  
  
  /**main as entry from any other Java program, gets the arguments as String, 
   * but does not catch unexpected exceptions and does not System.exit(...), use it to execute in a Java environment. 
   * @param sArgs the arguments as given in {@link #main(String[])}
   * @param logHelp maybe "System.out"
   * @param logError maybe "System.err" used for argument error messages first
   * @return 0 if all is ok
   * @throws Exception if unexpected.
   */
  public static int smain ( String[] sArgs, Appendable logHelp, Appendable logError) {
    MainArgs args = new MainArgs();
    if(sArgs.length ==0) {
      args.showHelp(logHelp);
      return(1);                // no arguments, help is shown.
    }
    if(  ! args.parseArgs(sArgs, logError)
      || ! args.testConsistence(logError)
      ) { 
      return(2);                    // argument error
    }
    //LogMessageStream log = new LogMessageStream(System.out);
    int exitCode = amain(args);
    return exitCode;
  }

  
  /**Main entry with outside prepared arguments
   * @param args
   * @return
   */
  public static int amain ( MainArgs args) {
    MainMoneySim thiz = new MainMoneySim(args);
    thiz.exec();
    return 0;
  }
  
  MainMoneySim (MainArgs args ) {
    this.args = args;
    File outFile = new File("output/out.txt");
    try {
      this.fsOut = new FileOutputStream(outFile);
    } catch (IOException e) {
      System.out.println("cannot create Log/Outfile: "+ outFile.getAbsolutePath());
    }
    this.log = new LogMessageStream(System.out, this.fsOut,  null, null, true, null);
    if(args.bVerbatile) {
      this.logAll = this.log;
    } else {
      this.logAll = new LogMessageStream(null, this.fsOut,  null, null, true, null);
    }
    this.readParameter = new ReadParameter(this.log);
    this.cityData = new CityData(this.log);
    this.evalShow = new EvalAndShow(this.log);
  }
  
  
  @Override protected void finalize () {
    if(this.fsOut !=null) { FileFunctions.close(this.fsOut); }
  }
  
  
  void exec () {
    this.readParameter.readParameter(new File("Parameter/SellingBehavior.txt"));
    SellerOnMarket[] allSeller = new SellerOnMarket[20];
    SellingBehavior parSeller = this.readParameter.idxSellingBehavior.get(("fish"));
    for(int ixSeller =0; ixSeller < allSeller.length; ++ixSeller) {
      allSeller[ixSeller] = new SellerOnMarket(ixSeller, parSeller, this.log);
    }
    //
    List<Family> listFamily = this.readParameter.createFamilies(this.cityData); // note: random usage depends on some parameter
    //
    this.readParameter.random.setSeed(this.args.seedRandom);     //<<<<<<<<<< start with a defined seed to get same results for amount
    MarketDay marketDay = new MarketDay(this.log, this.logAll);
    for(int day = 1; day < 10; ++day) {
      for(SellerOnMarket seller: allSeller) {
        float rand = this.readParameter.random.nextFloat();
        float amount = 4 * this.args.amountFish * rand * rand;  // on random 0.5 it is 250 kg, no more than 1000 kg
        seller.setMarketDayStart(amount);
      }
      try {
        this.log.writef("\n======================================================================================");
        this.log.writef("\n  A new market day %d with a nice morning, many fishes offer on market\n\n", day);
        marketDay.startMarket(allSeller);
        marketDay.calculate(listFamily, day);
        this.evalShow.outSellerMoney(allSeller);
        for(SellerOnMarket seller: allSeller) {
          seller.calcAfterMarketDay();
        }
          
        this.log.writef("\n\n");
      } catch (IOException e) {
        // TODO Auto-generated catch block
        e.printStackTrace();
      }
    }
    this.evalShow.outMeanFamilySituation(this.cityData);
    //
    if(this.fsOut !=null) { FileFunctions.close(this.fsOut); }
    Debugutil.stop();
  }
  
  
  
  
}
