package org.vishia.MoneySim;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.vishia.msgDispatch.LogMessage;
import org.vishia.util.Debugutil;
import org.vishia.util.StringFormatter;
import org.vishia.util.StringFunctions_C;

/**
 * This class contains operations for the market proceedings
 * <ul>
 * <li>{@link #calculate(SellerOnMarket[], List, int)} calculates all transactions on the market day between seller and shopper.
 * </ul>
 */
public class MarketDay {

  final LogMessage log, logAll;
  
  
  int ctShopper = 0, ctShopped =0;
  int ctNotOnMarket = 0;
  float timeOfday = 0;
  List<Family> listShopperWaiting = new LinkedList<>();
  List<Family> listShopperSearching = new LinkedList<>();

  SellerOnMarket[] allSeller;
  
  public MarketDay(LogMessage log, LogMessage logAll) {
    super();
    this.log = log;
    this.logAll = logAll;
  }


  public void startMarket (SellerOnMarket[] allSeller) {
    this.ctShopper = 0; this.ctShopped=0;
    this.ctNotOnMarket = 0;
    this.timeOfday = 0;
    this.listShopperWaiting.clear();
    this.listShopperSearching.clear();
    this.allSeller = allSeller;
  }


  public void calculate (List<Family> listFamily, int nrDay) throws IOException {
    //
    int zFamily = listFamily.size();
    float timeperShopper = 8.0f * this.allSeller.length / zFamily;
    float minPerShopper = timeperShopper * 60;
    Iterator<Family> iterShopper = listFamily.iterator();
    while(this.timeOfday < 8.0f) {                    //========vv main loop over one market day, 8 hour
      this.timeOfday += 10.0/60.0f;                   // step of 10 min
      int timeHour = (int)this.timeOfday;
      int timeMin = (int)(60 * (this.timeOfday - timeHour));
      int zShopper10min = zFamily / (8*6);  // all regarded, in 8h = 8*6* 10min
      //======>>>>
      addShopperSearching(zShopper10min, iterShopper);     // some shopper comes from iterShopper for the next 10 min
      //======>>>>
      shopperLooksforSellers();                   // all shopper looks for all the sellers first whether buy or not.
      //
      outputSituationShopperSellerBefore10minStep(timeHour, timeMin);
      shopperSellerTransaction();
      outputSituationShopperSellerAfter10minStep();
      for(SellerOnMarket seller: this.allSeller) {
        seller.calcNewPrice(this.timeOfday);
      }
    }
    for(Family family : listFamily) {            //--------vv increment the appetite to fish per day
      family.fish.wantToHave += 1.0f / family.fish.dayDiffNeed;  // add 1/7 per day generally, want fish again the next days
    }
    this.log.writef("\nsum families=%d, sum shopper=%d, sum shopped=%d sumNotShopped=%d", listFamily.size(), this.ctShopper, this.ctShopped, this.ctNotOnMarket);
    
    Debugutil.stop();
  }
  
  
  void addShopperSearching (int zShopper, Iterator<Family> iterShopper) {
    int zShopper10min = zShopper;
    
    while(--zShopper10min >0) {
      if(iterShopper.hasNext()) {
        Family shopper = iterShopper.next();
        if(shopper.fish.wantToHave >= 0.5f) {   // only if they want to have fish again after days.
          this.listShopperSearching.add(shopper);
          this.ctShopper +=1;
        } else {
          this.ctNotOnMarket +=1;                     // families which are not going to the market.
        }
      }
    }
  }

  
  
  void shopperLooksforSellers() {
    for(Family shopper : this.listShopperSearching) {
      int ixSellerFound = -1;
      float score = 999;
      for(int ixSeller = 0; ixSeller < this.allSeller.length; ++ixSeller) {
        SellerOnMarket seller= this.allSeller[ixSeller];
        float priceShopper = shopper.fish.nomPrice * shopper.fish.wantToHave;
        float score1 = seller.price / priceShopper + seller.listShopperWaiting.size() / shopper.nomQueueLengh;
        if(score1 < score && seller.amount > shopper.fish.amountPerUse) {
          score = score1; ixSellerFound = ixSeller;
        }
      }
      if(score < 1.5f) {
        SellerOnMarket seller= this.allSeller[ixSellerFound];
        seller.listShopperWaiting.add(shopper);
      } else {
        this.listShopperWaiting.add(shopper);
      }
    } // for 10 min some shopper comes to sellers
    this.listShopperSearching.clear();
    this.listShopperSearching.addAll(this.listShopperWaiting);
    this.listShopperWaiting.clear();
  }
  
  
  
  void outputSituationShopperSellerBefore10minStep (int timeHour, int timeMin) throws IOException {
    StringFormatter sfShopperQueueBefore = new StringFormatter();
    sfShopperQueueBefore.addint(timeHour, "11").add(':').addint(timeMin, "11").add(" Searching:").addint(this.listShopperSearching.size(),"22221").add(" Shopping|");
    //StringFormatter sf2 = new StringFormatter(); sf2.add("                   price: " );
    StringBuilder sb2 = new StringBuilder("                      price: " );
    StringBuilder sb0 = new StringBuilder("                     amount: " );
    for(SellerOnMarket seller: this.allSeller) {
      int nrShopperInQueue = seller.listShopperWaiting.size();
      int nrPoor=0, nrRich=0, nrMid=0;
      for(Family shopper: seller.listShopperWaiting) {
        if(shopper.nomQueueLengh < 5) {nrRich +=1; }
        else if(shopper.nomQueueLengh > 15) {nrPoor +=1; }
        else {nrMid +=1; }
      }
      if(nrShopperInQueue >0) {
//        sfShopperQueueBefore.addint(nrShopperInQueue,"   11   ");
        if(nrPoor >0) { sfShopperQueueBefore.addint(nrPoor,"11/"); } else {sfShopperQueueBefore.add("  /"); }
        if(nrMid >0) { sfShopperQueueBefore.addint(nrMid,"11/"); } else {sfShopperQueueBefore.add("  /"); }
        if(nrRich >0) { sfShopperQueueBefore.addint(nrRich,"11|"); } else {sfShopperQueueBefore.add("  |"); }
      } else {
        sfShopperQueueBefore.add("        |");
      }
      //sfShopperQueueBefore.addint(seller.listShopperWaiting.size(),"   11   ");
      StringFunctions_C.appendIntPict(sb0, (long)(seller.amount), " |2221 kg");
      sb2.append(" | $"); StringFunctions_C.appendIntPict(sb2, (long)(100*seller.price), "21.11");
      //sf2.add(" $").addFloat(seller.price, 2, 2);
      //======>>>>      =================================<< process the shopping with the shopper queue
 
    }      
    this.logAll.writef("\n%s",sb0);
    this.logAll.writef("\n%s",sb2);
    this.logAll.writef("\n%s",sfShopperQueueBefore.getContent());
      
  }
  
  
  /**
   * Calls the shopper transaction on sellers for the 10 min step.
   * It calls internally {@link SellerOnMarket#processShopper(float)}.
   */
  void shopperSellerTransaction () {
    for(SellerOnMarket seller: this.allSeller) {
      int nrShopped = seller.listShopperWaiting.size();     // all shopper waiting
      List<Family> listShopperNoSell = seller.processShopper(this.timeOfday);
      nrShopped -= listShopperNoSell.size();                // minus shopper which have not bought
      nrShopped -= seller.listShopperWaiting.size();        // minus shopper still waiting
      assert(nrShopped >=0);
      this.listShopperSearching.addAll(listShopperNoSell);  // all shopper which have not get the good because it was out.
      this.ctShopped += nrShopped;
    }    
  }
  

  /**
   * Shows reaming shopper queues on sellers. 
   * Outputs nothing if there are no remaining shoppers in the queues on sellers.
   * The queues are not empty if the time is out to sell.
   */
  void outputSituationShopperSellerAfter10minStep () {
    StringFormatter sfShopperQueueAfter = new StringFormatter();
    sfShopperQueueAfter.add(" Queue After 10 min ").pos(30);
    boolean bQueueAfter = false;
    for(SellerOnMarket seller: this.allSeller) {
      int nrShopperInQueueAfter = seller.listShopperWaiting.size();
      if(nrShopperInQueueAfter >0) {
        sfShopperQueueAfter.addint(nrShopperInQueueAfter,"    11   ");
      } else {
        sfShopperQueueAfter.add("         ");
      }
      bQueueAfter |= nrShopperInQueueAfter >0;
    }
    //this.log.writef("\n%s",sf2.getContent());
    if(bQueueAfter) { this.logAll.writef("\n%s",sfShopperQueueAfter.getContent()); }
    //this.log.writef("\n%d:%2d Searching: %d Shopping %s", timeHour, timeMin, listShopperSearching.size(), sQueueShopper);
    
  }
  

  
  
}
