package org.vishia.MoneySim;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.text.ParseException;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.TreeMap;

import org.vishia.msgDispatch.LogMessage;
import org.vishia.util.ExcUtil;
import org.vishia.util.FileFunctions;
import org.vishia.util.StringPartScanLineCol;

public class ReadParameter {

  Map<String, SellingBehavior> idxSellingBehavior = new TreeMap<>();
  
  Map<String, BuyingBehav> idxBuyingBehavior = new TreeMap<>();
  
  final LogMessage log;
  
  StringPartScanLineCol sc;
  
  String lineNext;
  
  Random random;
  

  
  String goodName = null;

  SellingBehavior.PriceOverTime[] arrayPriceOverTime;
  
  SellingBehavior.InitialAmountToPrice[] arrayInitialAmount2Price;
  
  enum WhatRead{nothing, hourAmountPrice}; // whatRead;

  
  
  public ReadParameter(LogMessage log) {
    super();
    this.log = log;
    this.random = new Random(1);
    this.random.setSeed(1);

  }
  
  

  /**Read all parameters from the given file.
   * @param file
   */
  void readParameter(File file) {
    BufferedReader rf = null;
    WhatRead whatRead = WhatRead.nothing;
    try { 
      rf = new BufferedReader(new FileReader(file));
      this.lineNext = "";
      do {
        String line = this.lineNext; this.lineNext = null;
        switch(whatRead) {
        case nothing: {
          if(line.startsWith("==good:")) { 
            prepareGood();
            this.goodName = line.substring(8).trim(); 
          } else if(line.startsWith("===PriceOverTime:")) { 
            readPriceOverTime(rf); 
          } else if(line.startsWith("===initalAmount2Price:")) { 
            readInitialAmount2Price(rf); 
          } else if(line.startsWith("==buy:")) { 
            readBuyingBehav(rf, line.substring(6)); 
          } else {
          }
        } break;
        case hourAmountPrice: {

        } break;

        } // switch whatRead
        if(this.lineNext == null) {
          this.lineNext = readLine(rf);
        }
      } while ( this.lineNext !=null);
      prepareGood();  // prepare the last good read.
      rf.close();
      rf = null;
    } catch(Exception exc) {
      CharSequence sError = ExcUtil.exceptionInfo("\nUNEXPECTED program error: ", exc, 0, 10);
      this.log.writef(sError.toString());
      if(rf !=null) { FileFunctions.close(rf); }// the necessary close
    }
    if(rf !=null) {  // this is only to prevent the warning 'resource leak, close'
      try { rf.close(); } catch (IOException e) {}
    }
  }
  
  
  
  
  /**If a new good line was found, prepare the last given good.
   * Do nothing if the first good was found. 
   * prepared good: Stored in {@link #idxSellingBehavior}
   * 
   * */
  void prepareGood () {
    if(this.goodName !=null) {
      if(this.arrayPriceOverTime !=null || this.arrayInitialAmount2Price !=null ) {
        if(this.arrayInitialAmount2Price ==null) {
          this.log.writef("\nFor the good '%s' the \"===initalAmount2Price:\" is missing", this.goodName);
        } else if(this.arrayPriceOverTime ==null) {
          this.log.writef("\nFor the good '%s' the \"===PriceOverTime:\" is missing", this.goodName);
        } else {
          SellingBehavior sellingBehavior = new SellingBehavior(this.goodName, this.arrayInitialAmount2Price, this.arrayPriceOverTime);
          this.idxSellingBehavior.put(this.goodName, sellingBehavior);
        }
      }
    }
  }
  
  
  
  private void readBuyingBehav (BufferedReader rf, String lineStart) throws IOException {
    readLinesOfBlock(rf, lineStart);
    List<BuyingBehav.PointPriceBuyingAccept> listPriceAccept = new LinkedList<>();
    String comment = null, good = null;
    try {
      if(this.sc.scanToAnyChar(":", '\0','\0','\0').scan(":").scanOk()) {
        comment = this.sc.getLastScannedString().trim();
      } else {
        this.log.writef("\n==buy: \"EXPECTED comment :\" missing, found: %s", this.sc.getCurrent(20)); 
      }
      if(this.sc.scan("good").scan("=").scanIdentifier().scan(";").scanOk() ) {
        good = this.sc.getLastScannedString();
      } else {
        this.log.writef("\n==buy: \"good= ...:\" missing, found: %s", this.sc.getCurrent(20)); 
      }
      while(this.sc.scanStart().scan("$").scanFloatNumber('.', false, "'").scan("=")
                .scanFloatNumber('.', false, "'").scanOk()) {
        float acceptance = (float)this.sc.getLastScannedFloatNumber();
        float price = (float)this.sc.getLastScannedFloatNumber();
        BuyingBehav.PointPriceBuyingAccept priceAccept = new BuyingBehav.PointPriceBuyingAccept(price, acceptance);
        listPriceAccept.add(priceAccept);
        if(!this.sc.scan(";").scanOk()) {
          this.log.writef("\nSyntay error readPriceOverTimeLine, expected: \"100.0 kg = $ 1.2;\" read line:\n  %d: \"%s\"", this.sc.getline(),this.sc.getCurrent(20));
          break;
        }
      }
    } catch (ParseException exc) {
      CharSequence sError = ExcUtil.exceptionInfo("\nUNEXPECTED program error: ", exc, 0, 10);
      this.log.writef(sError.toString());
    }
    if(listPriceAccept.size() ==0) {
      this.log.writef("\nUncomplete data for \"==buy ... %d: \"%s\"", this.sc.getline(), this.sc.getCurrent(20));
    } else {
      BuyingBehav.PointPriceBuyingAccept[] array = new BuyingBehav.PointPriceBuyingAccept[listPriceAccept.size()];
      listPriceAccept.toArray(array);
      BuyingBehav buyingBehav = new BuyingBehav(good, comment, array);
      this.idxBuyingBehavior.put(comment, buyingBehav);
    }
  }
  
  
  
  private void readInitialAmount2Price (BufferedReader rf) throws IOException {
    readLinesOfBlock(rf, null);
    List<SellingBehavior.InitialAmountToPrice> listAmount2price = new LinkedList<>();
    try {
      while(this.sc.scanStart().scanFloatNumber('.', false, "'").scan("kg").scan("=")
                .scan("$").scanFloatNumber('.', false, "'").scanOk()) {
        float price = (float)this.sc.getLastScannedFloatNumber();
        float amount = (float)this.sc.getLastScannedFloatNumber();
        SellingBehavior.InitialAmountToPrice amountPrice = new SellingBehavior.InitialAmountToPrice(amount, price);
        listAmount2price.add(amountPrice);
        if(!this.sc.scan(";").scanOk()) {
          this.log.writef("\nSyntay error readPriceOverTimeLine, expected: \"100.0 kg = $ 1.2;\" read line:\n  %d: \"%s\"", this.sc.getline(),this.sc.getCurrent(20));
          break;
        }
      }
    } catch (ParseException exc) {
      CharSequence sError = ExcUtil.exceptionInfo("\nUNEXPECTED program error: ", exc, 0, 10);
      this.log.writef(sError.toString());
    }
    if(listAmount2price.size() ==0) {
      this.log.writef("\nUncomplete data for readPriceOverTimeLine:\n  %d: \"%s\"", this.sc.getline(), this.sc.getCurrent(20));
    } else {
      this.arrayInitialAmount2Price = new SellingBehavior.InitialAmountToPrice[listAmount2price.size()];
      listAmount2price.toArray(this.arrayInitialAmount2Price);
      
    }
  }
  
  

  private void readPriceOverTime (BufferedReader rf ) throws IOException {
    readLinesOfBlock(rf, null);
    List<SellingBehavior.PriceOverTime> listPriceOverTime = new LinkedList<>();
    SellingBehavior.PriceOverTime priceOverTimeLine;
    while( this.sc.seekNoWhitespace().length() >0 && (priceOverTimeLine = readPriceOverTimeLine(this.sc)) !=null) {
      listPriceOverTime.add(priceOverTimeLine);
    }
    if(listPriceOverTime.size()==0) {
      
    } else {
      this.arrayPriceOverTime = new SellingBehavior.PriceOverTime[listPriceOverTime.size()];
      listPriceOverTime.toArray(this.arrayPriceOverTime);
    }
  }
  
  
  
  
  private SellingBehavior.PriceOverTime readPriceOverTimeLine (StringPartScanLineCol sc) {
    float timeOnSelling = Float.NaN;
    List<SellingBehavior.PriceAmountOnTime> listAmount2price = new LinkedList<>();
    try {
      if(!sc.scanStart().scanFloatNumber('.', false, "'").scan("::").scanOk()) {
        this.log.writef("\nSyntay error readPriceOverTimeLine, expected: \"1.0::...\", read line:\n  %d: \"%s\"", sc.getline(),sc.getCurrent(20));
      } else {
        timeOnSelling = (float)sc.getLastScannedFloatNumber();
        while(sc.scanStart().scanFloatNumber('.', false, "'").scan(":").scanFloatNumber('.', false, "'").scanOk()) {
          float price = (float)sc.getLastScannedFloatNumber();
          float amount = (float)sc.getLastScannedFloatNumber();
          SellingBehavior.PriceAmountOnTime amountPrice = new SellingBehavior.PriceAmountOnTime(amount, price);
          listAmount2price.add(amountPrice);
          if(sc.scan(";;").scanOk()) {
            break;
          } else if(!sc.scan(";").scanOk()) {
            this.log.writef("\nSyntay error readPriceOverTimeLine, expected: \"0.8:1.2;\" read line:\n  %d: \"%s\"", sc.getline(),sc.getCurrent(20));
            break;
          }
        } 
      }
    } catch (ParseException exc) {
      CharSequence sError = ExcUtil.exceptionInfo("\nUNEXPECTED program error: ", exc, 0, 10);
      this.log.writef(sError.toString());
    }
    SellingBehavior.PriceAmountOnTime[] arrayAmount2Price = new SellingBehavior.PriceAmountOnTime[listAmount2price.size()];
    listAmount2price.toArray(arrayAmount2Price);
    if(timeOnSelling == Float.NaN || listAmount2price.size() ==0) {
      this.log.writef("\nUncomplete data for readPriceOverTimeLine:\n  %d: \"%s\"", sc.getline(),sc.getCurrent(20));
      return null;
    } else {
      return new SellingBehavior.PriceOverTime(timeOnSelling, arrayAmount2Price);
    }
  }

  /**Reads one line from file and trims comment and leading and trailing spaces-
   * @param rf BufferedReader from file
   * @return maybe an empty line "", null on end of file.
   * @throws IOException
   */
  private String readLine (BufferedReader rf) throws IOException {
    String line;
    while ( (line = rf.readLine()) !=null) {
      line = line.trim();
      int posComment = line.indexOf("##");
      if(posComment >0) { line = line.substring(0,posComment).trim(); }
      if(posComment !=0 && line.length()>0) {
        return line;
      }
    }
    return null;
  }

  void readLinesOfBlock (BufferedReader rf, String lineStart) throws IOException {
    String line;
    String lines= lineStart ==null ? "" : lineStart;
    while ((line = readLine(rf)) !=null) {
      if(line.startsWith("==")) {
        this.lineNext = line;                       // the next already read line, not processed here
        break;
      }
      lines += line + "\n";
    }
    this.sc = new StringPartScanLineCol(lines);
    this.sc.setIgnoreWhitespaces(true);
  }
  
  
  List<Family> createFamilies (CityData city) {
    List<Family> listFamily= new LinkedList<>();
    city.families = new Family[4][];
    city.families[0] = new Family[3000];
    int nrFamily = 0;
    for(int ix = 0; ix < city.families[0].length; ++ix) {  // add some families
      Family family = new Family("rich", null, +nrFamily);
      family.fish = new GoodFromMarket();
      family.fish.amountPerUse = 5 * 0.5f;  // 5 people
      family.fish.nomPrice = 12.0f;
      family.nomQueueLengh = 3;
      family.fish.dayDiffNeed = 7;
      family.fish.wantToHave = this.random.nextFloat();
      family.fish.buyingBehavior = this.idxBuyingBehavior.get("fish rich family");
      family.money = 10000.0f;
      city.families[0][ix] = family;
      listFamily.add(family);
    }
    city.families[1] = new Family[4000];
    for(int ix = 0; ix < city.families[1].length; ++ix) {  // add some families
      Family family = new Family("normal1", null, +nrFamily);
      family.fish = new GoodFromMarket();
      family.fish.amountPerUse = 5*0.3f;  // 5 people
      family.fish.nomPrice = 8.0f;
      family.nomQueueLengh = 10;
      family.fish.dayDiffNeed = 7;
      family.fish.wantToHave = this.random.nextFloat();
      family.fish.buyingBehavior = this.idxBuyingBehavior.get("fish normal family");
      family.money = 100.0f;
      city.families[1][ix] = family;
      listFamily.add(family);
    }
    city.families[2] = new Family[3000];
    for(int ix = 0; ix < city.families[2].length; ++ix) {  // add some families
      Family family = new Family("normal2", null, +nrFamily);
      family.fish = new GoodFromMarket();
      family.fish.amountPerUse = 5*0.3f;  // 5 people
      family.fish.nomPrice = 12.0f;
      family.nomQueueLengh = 12;
      family.fish.dayDiffNeed = 3;
      family.fish.wantToHave = this.random.nextFloat();
      family.fish.buyingBehavior = this.idxBuyingBehavior.get("fish normal family");
      family.money = 1000.0f;
      city.families[2][ix] = family;
      listFamily.add(family);
    }
    city.families[3] = new Family[10000];
    for(int ix = 0; ix < city.families[3].length; ++ix) {  // add some families
      Family family = new Family("poor", null, +nrFamily);
      family.fish = new GoodFromMarket();
      family.fish.amountPerUse = 5 * 0.16f;  // 5 people
      family.fish.nomPrice = 4.0f;
      family.nomQueueLengh = 30;
      family.fish.dayDiffNeed = 7;
      family.fish.wantToHave = this.random.nextFloat();
      family.fish.buyingBehavior = this.idxBuyingBehavior.get("fish poor family");
      family.money = 100.0f;
      city.families[3][ix] = family;
      listFamily.add(family);
    }
    // now mix the consumer because they are too strong sorted:
    { @SuppressWarnings("unchecked") List<Family>[] listsMix = new List[100];
      for(int ix = 0; ix < listsMix.length; ++ix) {
        listsMix[ix] = new java.util.LinkedList<Family>();
      }
      for(Family consumer: listFamily) {           //--------vv mix in different Lists.
        int ixSort = (int)(100*this.random.nextFloat());
        listsMix[ixSort].add(consumer);
      }
      listFamily.clear();
      for(List<Family> listFamily1 : listsMix ) {
        for(Family family: listFamily1) {
          listFamily.add(family);                          // now it is a random order of members.
        }
      }
    }
    city.familiesUnsorted = listFamily;
    return listFamily;
  }
  
}
