package org.vishia.MoneySim;

import java.util.LinkedList;
import java.util.List;

import org.vishia.msgDispatch.LogMessage;

/**
 * This class describes a good which is offer on the market.
 */
public class SellerOnMarket {
  
  
  final int nr;
  
  final LogMessage log;
  
  /**
   * The current available amount of the good.
   */
  float amountStart, amount;
  
  float amountRelatLastPriceCalc;
  
  float priceStart, price;
  
  float moneySum, moneyOfDay;
  
  List<Family> listShopperWaiting = new LinkedList<>();
  
  /**
   * The current time of day (hour) for selling on market. 
   * If nothing was sold, the price should be go lower and vice versa.
   * This function needs the time.
   */
  float time, timeLastPriceCalc;
  
  int ixCurrPriceOverTime;
  
  
  
  public SellerOnMarket( int nr, SellingBehavior par, LogMessage log
  ) {
    super();
    this.nr = nr;
    this.par = par;
    this.log = log;
  }

  
  public void setMarketDayStart (float amount) {
    this.amount = this.amountStart = amount;
    this.amountRelatLastPriceCalc = 1.0f;
    this.price = this.priceStart = MCalc.round2(MCalc.getInterpolation(this.par.initialAmountToPrice, this.amount));
    this.time = 0; this.timeLastPriceCalc = 0;
    this.ixCurrPriceOverTime = 0;
    
  }
  
  
  public List<Family> processShopper (float timeEnd) {
    int ctShopped = 0;
    List<Family> listShopperNoSell = new LinkedList<>();
    
    while(this.listShopperWaiting.size() >0 && this.time < timeEnd) {
      Family shopper = this.listShopperWaiting.remove(0);
      float amount = shopper.fish.amountPerUse;  // TODO multiply more for self-store
      if(this.amount >= amount) {
        ctShopped +=1;
        //-------------------------------------------------vv seller: increment money, decrement amount
        this.amount -= amount;
        float price = amount * this.price;
        this.moneyOfDay += price;
        shopper.fish.amountConsumed += amount;
        shopper.money -= price;
        //-------------------------------------------------vv shopper: decrement wantToHave
        shopper.fish.wantToHave -= amount / shopper.fish.amountPerUse;  // I have it, need no more
        //
        this.time += (float)Math.sqrt(amount) * 0.5f/60.0f;        // 0.5 minutes per selling operation for 1 kg, for 10 kg it is 1.5 min
      } else {
        listShopperNoSell.add(shopper);   // add again, cannot sell
      }
    }
    return listShopperNoSell;
  }
  
  /**
   * The parameter which controls selling. 
   * Hint: The parameter may be also variable for deeper behavior simulation,
   * But for a simple simulation solution they are constants read from textual files. 
   */
  SellingBehavior par;
  
  public void calcNewPrice (float timeOfDay) {
    if( this.ixCurrPriceOverTime < this.par.priceOverTime.length-2
     && timeOfDay > this.par.priceOverTime[this.ixCurrPriceOverTime+1].timeOnSelling
      ) {
      this.ixCurrPriceOverTime +=1;                        // use next entry of price over time table
    }
    SellingBehavior.PriceAmountOnTime[] table1 = this.par.priceOverTime[this.ixCurrPriceOverTime+1].price4Amount;
    float amountRelative = this.amount / this.amountStart;
    float timeInHour = timeOfDay - this.par.priceOverTime[this.ixCurrPriceOverTime].timeOnSelling; // 0.166 ... 1.0 step in 10 min
    float amountRelatDiff10min = amountRelative - this.amountRelatLastPriceCalc;   // change during the last 10 min
    float amountRelatDiffFullHour = amountRelatDiff10min * 6 * (1.0f-timeInHour);  // extrapolate change from last till full hour
    float amountRelative1Hour = amountRelatDiffFullHour + amountRelative;          // extrapolate relative amount to the next full hour
    float diffPriceRelative1Hour = MCalc.getInterpolation(table1, amountRelative1Hour);  // price from extrapolate on next full hour
    float diffPriceRelative10min = (diffPriceRelative1Hour - 1.0f)/6 + 1.0f;      // relative price change for this step
    float newPrice = this.price * diffPriceRelative10min;
//    this.log.writef("\nSeller %d new price, selled %2d%% from %2.1f, old price: $%2.2f / $%2.2f, new price $%2.2f"
//                       , this.nr, (int)(100* amountRelative), this.amountStart, this.priceStart, this.price, newPrice);
    this.price = newPrice;
    this.amountRelatLastPriceCalc = amountRelative;
  }
  
  
  public void calcAfterMarketDay () {
    this.moneySum += this.moneyOfDay;
    this.moneyOfDay = 0;
  }
}
