package org.vishia.MoneySim;

/**
 * This class contains the parameter which determines the behavior of selling on market. 
 * An instance of this class is associated to a {@link SellerOnMarket} which is related to only one good to sell.
 */
public class SellingBehavior {

  
  String nameGood;
  
  /**This array is sorting to the price in {@link PointPriceBuyingAccept#price}
   * and contains a curve for the buying acceptance.
   * Both values are nominal values to 1.0. It means 1.0 for the price is the normal expected price for one unit.
   */
  InitialAmountToPrice[] initialAmountToPrice;

  /**
   * This is the near most important function. If the time of selling day is progressed, and less is sold,
   * the price should go lower. In opposite, if much is sold in the first hours, the price should rise up.
   */
  PriceOverTime[] priceOverTime;

  
  
  public SellingBehavior( String nameGood
  , InitialAmountToPrice[] initialAmountToPrice, PriceOverTime[] priceOverTime
  ) {
    super();
    this.nameGood = nameGood;
    this.initialAmountToPrice = initialAmountToPrice;
    this.priceOverTime = priceOverTime;
  }



  static class InitialAmountToPrice extends MCalc.ValuePair {
    /**
     * One point of the curve for the initial price given by the amount of goods.
     * General, if a lot of goods are available, is is not expected that all would be bought. 
     * Hence the price should be lower. That is also ok because a higher amount of peaces may be bought in any act of purchase.
     * Means the related effort is also lower.
     * <br>
     * On the other side, if a less amount of peaces is given, the demand may be high, and a higher price reduces cheap consumer to buy too much goods
     * This is a point of a curve from experience. It can be variable by improving experience.
     */
    //final float price;
    /**
     * Acceptance to buy a good for the given price. It should be near 1.0 for the price 1.0,
     * can be fast get lower if a higher price is not accepted,
     * and get be higher for a lower price. But this depends of the quality of the good:
     * Is it able to store at home, make durable goods from it, or such. 
     */
    //final float amount;
    
    public InitialAmountToPrice ( float amount, float price ) {
      super(amount, price);
    }
    
    @Override public String toString () {
      return "" + super.x + " kg = $" + super.y;
    }

  }
  
  
  /**
   * One point for {@link PriceOverTime}, it is depending of the amount: 
   * On a definite time of selling the relative amount should influence the price,
   * If less is sold, the price should go lower. 
   * In opposite, if much is sold, less available, the price should go higher.
   */
  static class PriceAmountOnTime extends MCalc.ValuePair {
    
    // y float priceRelative;
    
    /**
     * The amount of peaces, but related to the amount on beginning the selling day.
     * It means it is a number between 0..1.0
     */
    // x float amountAvail;

    public PriceAmountOnTime ( float amountAvail, float priceRelative ) {
      super(amountAvail, priceRelative);
    }
    
    @Override public String toString () {
      return "" + 100.0f * super.x + "% : " + super.y;
    }
  }
  
  
  
  /**
   * One point of the near most important function for selling: 
   * If the time of selling day is progressed, and less is sold,
   * the price should go lower. In opposite, if much is sold in the first hours, the price should rise up.
   */
  static class PriceOverTime {
    
    float timeOnSelling;

    PriceAmountOnTime[] price4Amount;
    
    public PriceOverTime ( float timeOnSelling, PriceAmountOnTime[] price4Amount ) {
      this.price4Amount = price4Amount;
      this.timeOnSelling = timeOnSelling;
    }
    
    @Override public String toString () {
      StringBuilder sb = new StringBuilder();
      sb.append(this.timeOnSelling).append("h :: ");
      for(PriceAmountOnTime point: this.price4Amount) {
        sb.append(point).append("; ");
      }
      sb.append(";;");
      return sb.toString();
    }

    
  }
  
  
  
  
}
